@@ -1,5 +1,6 @@
 from .api_sync import UnauthorizedAPI
-from .sansio import ApiSig
+from .models import _RaiseIfError
+from ._sansio import ApiSig
 
 _AUTHORIZATION_URL = 'https://www.rememberthemilk.com/services/auth/'
 
@@ -8,10 +9,10 @@ class AuthorizationSession:
 
 	def __init__(self, apiKey, sharedSecret, perms):
 		self._api = UnauthorizedAPI(apiKey, sharedSecret)
-		self._frob = self._api.AuthGetFrob()
+		self._frob = _RaiseIfError(self._api.AuthGetFrob())
 		params = {'api_key': apiKey, 'perms': perms, 'frob': self._frob}
 		params['api_sig'] = ApiSig(sharedSecret, params)
 		self.url = _AUTHORIZATION_URL + '?' + '&'.join([f'{k}={v}' for k, v in params.items()])
 
 	def Done(self):
-		return self._api.AuthGetToken(self._frob)
+		return _RaiseIfError(self._api.AuthGetToken(self._frob))
