@@ -1,11 +1,13 @@
+from __future__ import annotations
+
 from datetime import datetime
 from enum import Enum, IntEnum
-from typing import Optional, Union
+from typing import Annotated
 
-from pydantic import AnyHttpUrl, BaseModel, Field, constr, validator # pylint: disable=no-name-in-module
-from pydantic.types import ConstrainedStr # pylint: disable=no-name-in-module
+from pydantic import BaseModel, Field, field_validator
+from pydantic.types import StringConstraints
 
-from .utils import EmptyStrToNone
+from ._utils import EmptyStrToNone
 
 class RTMError(Exception):
 	def __init__(self, code, message):
@@ -21,18 +23,23 @@ class ErrorData(BaseModel):
 	msg: str
 
 class OkStat(BaseModel):
-	stat: constr(regex='ok')
+	stat: Annotated[str, StringConstraints(pattern='ok')]
 
 class FailStat(BaseModel):
-	stat: constr(regex='fail')
+	stat: Annotated[str, StringConstraints(pattern='fail')]
 	err: ErrorData
 
+def _RaiseIfError(result):
+	if isinstance(result, FailStat):
+		raise RTMError(result.err.code, result.err.msg)
+	return result
+
 class EchoResponse(OkStat):
 	__test__ = False # avoid pytest warning
-	method: constr(regex='rtm.test.echo')
+	method: Annotated[str, StringConstraints(pattern='rtm.test.echo')]
 
 class RTMList(BaseModel):
-	id: int
+	id: str
 	name: str
 	deleted: bool
 	locked: bool
@@ -41,7 +48,8 @@ class RTMList(BaseModel):
 	smart: bool
 
 	@classmethod
-	@validator('smart')
+	@field_validator('smart')
+	@classmethod
 	def NotASmartList(cls, value: bool) -> bool:
 		if value is not False:
 			raise ValueError('Must be False for non-smart lists')
@@ -51,17 +59,18 @@ class RTMSmartList(RTMList):
 	filter: str
 
 	@classmethod
-	@validator('smart')
+	@field_validator('smart')
+	@classmethod
 	def IsASmartList(cls, value: bool) -> bool:
 		if value is not True:
 			raise ValueError('Must be True for smart lists')
 		return value
 
 class ListPayload(BaseModel):
-	list: list[Union[RTMSmartList, RTMList]]
+	list: list[RTMSmartList | RTMList]
 
 class SingleListResponse(OkStat):
-	list: Union[RTMSmartList, RTMList]
+	list: RTMSmartList | RTMList
 
 class ListsResponse(OkStat):
 	lists: ListPayload
@@ -100,17 +109,17 @@ class PriorityDirectionEnum(Enum):
 class Task(BaseModel):
 	id: str
 	added: datetime
-	completed: Optional[EmptyStrToNone[datetime]]
-	deleted: Optional[EmptyStrToNone[datetime]]
-	due: Optional[EmptyStrToNone[datetime]]
+	completed: EmptyStrToNone[datetime | None]
+	deleted: EmptyStrToNone[datetime | None]
+	due: EmptyStrToNone[datetime | None]
 	estimate: str
 	has_due_time: bool
 	has_start_time: bool
 	postponed: int
 	priority: PriorityEnum
-	start: Optional[EmptyStrToNone[datetime]]
+	start: EmptyStrToNone[datetime | None]
 
-class NotesResponsePayload(BaseModel):
+class Note(BaseModel):
 	id: str
 	created: datetime
 	modified: datetime
@@ -123,10 +132,10 @@ class Transaction(BaseModel):
 
 class NotesResponse(OkStat):
 	transaction: Transaction
-	note: NotesResponsePayload
+	note: Note
 
 class NotePayload(BaseModel):
-	note: list[NotesResponsePayload]
+	note: list[Note]
 
 class Tags(BaseModel):
 	tag: list[str]
@@ -137,18 +146,18 @@ class TaskSeries(BaseModel):
 	modified: datetime
 	name: str
 	source: str
-	url: EmptyStrToNone[AnyHttpUrl]
+	url: EmptyStrToNone[str | None]
 	location_id: str
 	participants: list[str]
 
 	# see test_that_unions_are_necessary_for_notes_list for why the Union is necessary
 	# in the case where this is a list[str], it's always an empty list
-	notes: Union[NotePayload, list[str]]
+	notes: NotePayload | list[str]
 	task: list[Task]
 
 	# see test_that_unions_are_necessary_for_tag_list for why the Union is necessary
 	# in the case where this is a list[str], it's always an empty list
-	tags: Union[Tags, list[str]]
+	tags: Tags | list[str]
 
 class TaskPayload(BaseModel):
 	id: str
@@ -161,12 +170,15 @@ class TaskResponse(OkStat):
 class TasksInListPayload(BaseModel):
 	id: str
 	# can be missing if there are no tasks in the list returned from TasksGetList with just a listid
-	taskseries: Optional[list[TaskSeries]]
+	taskseries: list[TaskSeries] | None = None
+
+# hack to make the module import
+ListOfTasksInListPayload = list[TasksInListPayload]
 
 class TaskListPayload(BaseModel):
 	rev: str
 	# if there are are no tasks in the list, returned from TasksGetList via a filter, this node is missing
-	list: Optional[list[TasksInListPayload]]
+	list: ListOfTasksInListPayload | None = None
 
 class TaskListResponse(OkStat):
 	tasks: TaskListPayload
@@ -206,18 +218,17 @@ class Topic(BaseModel):
 class TopicListResponse(OkStat):
 	topics: Topic
 
-class SubscriptionData:
+class SubscriptionPayload(BaseModel):
 	id: str
 	url: str
-	format: ConstrainedStr('json')
+	format: Annotated[str, StringConstraints(pattern='json')]
 	expires: datetime
 	pending: bool
-	topics: list[str]
+	topics: Topic | list[str]
 
-class SubscriptionPayload(BaseModel, SubscriptionData):
-	pass
+# SubscriptionPayload.model_rebuild()
 
-class SubscriptionResponse(OkStat, SubscriptionData):
+class SubscriptionResponse(OkStat):
 	transaction: Transaction
 	subscription: SubscriptionPayload
 
@@ -225,4 +236,4 @@ class SubscriptionList(BaseModel):
 	subscription: list[SubscriptionPayload]
 
 class SubscriptionListResponse(OkStat):
-	subscriptions: Union[SubscriptionList, list[SubscriptionPayload]]
+	subscriptions: SubscriptionList | list[SubscriptionPayload]
